@@ -1,19 +1,19 @@
-from contextlib import contextmanager
+from contextlib import contextmanager, suppress
 from dataclasses import replace
 from functools import lru_cache
 from types import new_class
 from typing import (
     Any,
     Collection,
-    Dict,
     Generic,
-    Hashable,
     Iterable,
+    Mapping,
     Optional,
     Sequence,
     Tuple,
     Type,
     TypeVar,
+    Union,
 )
 
 from apischema.conversions import LazyConversion
@@ -26,23 +26,22 @@ from apischema.conversions.conversions import (
     is_identity,
     resolve_any_conversion,
 )
-from apischema.conversions.dataclass_models import handle_dataclass_model
 from apischema.conversions.utils import is_convertible
 from apischema.metadata.implem import ConversionMetadata
+from apischema.metadata.keys import CONVERSION_METADATA
 from apischema.type_names import type_name
 from apischema.types import AnyType
-from apischema.typing import get_args
+from apischema.typing import get_args, is_type_var
 from apischema.utils import (
-    Lazy,
     context_setter,
+    get_args2,
     get_origin_or_type,
     has_type_vars,
-    is_hashable,
     is_subclass,
     substitute_type_vars,
     subtyping_substitution,
 )
-from apischema.visitor import Result, Visitor
+from apischema.visitor import Result, Unsupported, Visitor
 
 Deserialization = ResolvedConversions
 Serialization = ResolvedConversion
@@ -52,7 +51,7 @@ Conv = TypeVar("Conv")
 class ConversionsVisitor(Visitor[Result], Generic[Conv, Result]):
     def __init__(self, default_conversion: DefaultConversion):
         self.default_conversion = default_conversion
-        self._conversions: Optional[AnyConversion] = None
+        self._conversion: Optional[AnyConversion] = None
 
     def _has_conversion(
         self, tp: AnyType, conversion: Optional[AnyConversion]
@@ -66,21 +65,32 @@ class ConversionsVisitor(Visitor[Result], Generic[Conv, Result]):
 
     def annotated(self, tp: AnyType, annotations: Sequence[Any]) -> Result:
         for annotation in reversed(annotations):
-            if isinstance(annotation, ConversionMetadata):
-                with self._replace_conversion(self._annotated_conversion(annotation)):
+            if isinstance(annotation, Mapping) and CONVERSION_METADATA in annotation:
+                with self._replace_conversion(
+                    self._annotated_conversion(annotation[CONVERSION_METADATA])
+                ):
                     return super().annotated(tp, annotations)
         return super().annotated(tp, annotations)
 
-    def _union_result(self, results: Iterable[Result]) -> Result:
+    def _union_results(self, alternatives: Iterable[AnyType]) -> Sequence[Result]:
+        results = []
+        for alt in alternatives:
+            with suppress(Unsupported):
+                results.append(self.visit(alt))
+        if not results:
+            raise Unsupported(Union[tuple(alternatives)])
+        return results
+
+    def _visited_union(self, results: Sequence[Result]) -> Result:
         raise NotImplementedError
 
     def union(self, alternatives: Sequence[AnyType]) -> Result:
-        return self._union_result(map(self.visit, alternatives))
+        return self._visited_union(self._union_results(alternatives))
 
     @contextmanager
     def _replace_conversion(self, conversion: Optional[AnyConversion]):
-        with context_setter(self) as setter:
-            setter._conversions = resolve_any_conversion(conversion)
+        with context_setter(self):
+            self._conversion = resolve_any_conversion(conversion) or None
             yield
 
     def visit_with_conv(
@@ -113,15 +123,15 @@ class ConversionsVisitor(Visitor[Result], Generic[Conv, Result]):
 
     def visit(self, tp: AnyType) -> Result:
         if not is_convertible(tp):
-            return self.visit_conversion(tp, None, False, self._conversions)
-        dynamic, conversion = self._has_conversion(tp, self._conversions)
+            return self.visit_conversion(tp, None, False, self._conversion)
+        dynamic, conversion = self._has_conversion(tp, self._conversion)
         if not dynamic:
             _, conversion = self._has_conversion(
                 tp, self.default_conversion(get_origin_or_type(tp))  # type: ignore
             )
         next_conversion = None
         if not dynamic and is_subclass(tp, Collection):
-            next_conversion = self._conversions
+            next_conversion = self._conversion
         return self.visit_conversion(tp, conversion, dynamic, next_conversion)
 
 
@@ -165,12 +175,14 @@ class DeserializationVisitor(ConversionsVisitor[Deserialization, Result]):
                     if get_args(tp):
                         wrapper = wrapper[get_args(tp)]
                     conv = ResolvedConversion(replace(conv, source=wrapper))
-                conv = handle_dataclass_model(conv)
-                _, substitution = subtyping_substitution(tp, conv.target)
-                source = substitute_type_vars(conv.source, substitution)
-                result.append(
-                    ResolvedConversion(replace(conv, source=source, target=tp))
-                )
+                if is_type_var(conv.source) or any(
+                    map(is_type_var, get_args2(conv.source))
+                ):
+                    _, substitution = subtyping_substitution(tp, conv.target)
+                    conv = replace(
+                        conv, source=substitute_type_vars(conv.source, substitution)
+                    )
+                result.append(ResolvedConversion(replace(conv, target=tp)))
         if identity_conv and len(result) == 1:
             return True, None
         else:
@@ -188,10 +200,11 @@ class DeserializationVisitor(ConversionsVisitor[Deserialization, Result]):
         dynamic: bool,
         next_conversion: Optional[AnyConversion],
     ) -> Result:
-        return self._union_result(
+        results = [
             self.visit_with_conv(conv.source, sub_conversion(conv, next_conversion))
             for conv in conversion
-        )
+        ]
+        return self._visited_union(results)
 
 
 class SerializationVisitor(ConversionsVisitor[Serialization, Result]):
@@ -204,10 +217,14 @@ class SerializationVisitor(ConversionsVisitor[Serialization, Result]):
             if is_subclass(tp, conv.source):
                 if is_identity(conv):
                     return True, None
-                conv = handle_dataclass_model(conv)
-                substitution, _ = subtyping_substitution(conv.source, tp)
-                target = substitute_type_vars(conv.target, substitution)
-                return True, ResolvedConversion(replace(conv, source=tp, target=target))
+                if is_type_var(conv.target) or any(
+                    map(is_type_var, get_args2(conv.target))
+                ):
+                    substitution, _ = subtyping_substitution(conv.source, tp)
+                    conv = replace(
+                        conv, target=substitute_type_vars(conv.target, substitution)
+                    )
+                return True, ResolvedConversion(replace(conv, source=tp))
         else:
             return False, None
 
@@ -226,36 +243,3 @@ class SerializationVisitor(ConversionsVisitor[Serialization, Result]):
         return self.visit_with_conv(
             conversion.target, sub_conversion(conversion, next_conversion)
         )
-
-
-class CachedConversionsVisitor(ConversionsVisitor[Conv, Result]):
-    def __init__(self, default_conversion: DefaultConversion):
-        super().__init__(default_conversion)
-        self._visit_cache: Dict[
-            Tuple[AnyType, Optional[AnyConversion], Hashable], Result
-        ] = {}
-
-    def _cache_key(self) -> Hashable:
-        return None
-
-    def _cache_result(self, lazy: Lazy[Result]) -> Result:
-        raise NotImplementedError
-
-    def visit(self, tp: AnyType) -> Result:
-        if not is_hashable(tp):
-            return super().visit(tp)
-        cache_key = (tp, self._conversions, self._cache_key())
-        if cache_key in self._visit_cache:
-            return self._visit_cache[cache_key]
-        result = None
-
-        def lazy_result():
-            assert result is not None
-            return result
-
-        self._visit_cache[cache_key] = self._cache_result(lazy_result)
-        try:
-            result = super().visit(tp)
-        finally:
-            del self._visit_cache[cache_key]
-        return result
